@@ -87,8 +87,8 @@ class Executor:
                 else:
                     logger.debug('    still alive; will try again soon')
 
-        self._instances = {}
-        self._jobs_to_instances = {}
+        self._instances.clear()
+        self._jobs_to_instances.clear()
 
         self._lock = None
 
@@ -119,7 +119,7 @@ class Executor:
             instance_id = self._gen_new_instance_id()
             process = self.mp_ctx.Process(target=run_job, name=f'Process-{instance_id}',
                                           args=(job, instance_id, run_time, self._event_queue),
-                                          kwargs=dynamic_args)
+                                          kwargs={**job.kwargs, **dynamic_args})
 
             self._instances[instance_id] = {
                 'job': job,
@@ -174,13 +174,27 @@ class Executor:
             except Empty:
                 continue
 
-            logger.debug(f'got event ({event}) from job {event.job_id}:{event.instance_id}; '
+            job = self._instances[event.instance_id]['job']
+
+            logger.debug(f'got event ({event}) from job {job.id}:{event.instance_id}; '
                          f'event queue size: {self._event_queue.qsize()}')
 
             if event.success:
-                logger.info(f'Job "{event.job_id}" executed successfully')
+                logger.info(f'Job "{job.id}" executed successfully')
+                if job.callback:
+                    try:
+                        job.callback(job.id, event.instance_id, event.retval)
+                    except Exception as e:
+                        logger.error(f'callback failed!')
+                        logger.exception(e)
             else:
-                logger.error(f'Job "{event.job_id}" raised an exception\n{event.formatted_exc}')
+                logger.error(f'Job "{job.id}" raised an exception\n{event.formatted_exc}')
+                if job.error_callback:
+                    try:
+                        job.error_callback(job.id, event.instance_id, event.exc)
+                    except Exception as e:
+                        logger.error(f'error callback failed!')
+                        logger.exception(e)
 
             self._job_cleanup(event.job_id, event.instance_id)
             self._scheduler._dispatch_event(event)
@@ -221,12 +235,16 @@ class Executor:
             logger.debug('cleaned up')
 
     def _gen_new_instance_id(self) -> int:
+        all_possible_ids = set(range(10000, 100000))
+
         with self._lock:
-            while (instance_id := random.randint(1000, 9999)) in list(self._instances.keys()) + self._recent_instance_ids:
-                pass
+            current_ids = set(self._instances.keys()) | set(self._recent_instance_ids)
+            available_ids = list(all_possible_ids - current_ids)
+
+            instance_id = random.choice(available_ids)
 
             self._recent_instance_ids.append(instance_id)
-            if len(self._recent_instance_ids) > 1000:
+            if len(self._recent_instance_ids) > 10000:
                 self._recent_instance_ids.pop(0)
 
             return instance_id
@@ -252,7 +270,8 @@ def run_job(job: Job, instance_id: int, run_time: datetime, event_queue: multipr
 
 
 class ExecutorNotSetupException(Exception):
-    pass
+    def __init__(self):
+        super().__init__('Executor not setup')
 
 
 class MaxJobInstancesReachedException(Exception):
