@@ -80,42 +80,43 @@ class SqlAlchemyJobStore(JobStore):
         self.engine.dispose()
 
     @db_user
-    def add_job(self, job: Job, replace_existing: True, session=None):
+    def add_job(self, job: Job, replace_existing: bool = True, session: Session = None):
         if self.contains_job(job.id):
             if not replace_existing:
-                raise JobAlreadyExistsException
+                raise JobAlreadyExistsException(job.id)
 
             self.update_job(job, session=session)
             return
 
-        if job.next_run_time is None:
+        if job.next_run_time is None and job.active:
             job.next_run_time = job.trigger.get_next_fire_time(None, None)
 
-        self._logger.debug(f'Add job ({job})')
+        self._logger.debug(f'Add job ({job}) next run time: {job.next_run_time}')
 
         new_job = DbJob(id=job.id,
-                        next_run_time=job.next_run_time.timestamp(),
+                        next_run_time=job.get_next_run_time_ts(),
                         job_state=pickle.dumps(job.__getstate__(), self.pickle_protocol))
         session.add(new_job)
 
     @db_user
-    def update_job(self, job: Job, session=None):
+    def update_job(self, job: Job, session: Session = None):
         existing_job = self._get_db_job(job.id, session)
 
-        if job.next_run_time is None:
+        if job.next_run_time is None and job.active:
             if existing_job.next_run_time is None:
                 job.next_run_time = job.trigger.get_next_fire_time(None, None)
             else:
                 job.next_run_time = datetime.fromtimestamp(existing_job.next_run_time, tz=self._scheduler.tz)
 
-        self._logger.debug(f'Update job ({job})')
+        self._logger.debug(f'Update job ({job}) next run time: {job.next_run_time}')
 
-        existing_job.next_run_time = job.next_run_time.timestamp()
+        existing_job.next_run_time = job.get_next_run_time_ts()
         existing_job.job_state = pickle.dumps(job.__getstate__(), self.pickle_protocol)
 
     @db_user
-    def remove_job(self, job_id: str, session=None):
+    def remove_job(self, job_id: str, session: Session = None):
         existing_job = self._get_db_job(job_id, session)
+        self._logger.debug(f'Delete job ({self._reconstitute_job(existing_job)})')
         session.delete(existing_job)
 
     @db_user
@@ -123,13 +124,13 @@ class SqlAlchemyJobStore(JobStore):
         session.query(DbJob).delete()
 
     @db_user
-    def get_job(self, job_id: str, session=None) -> Job:
+    def get_job(self, job_id: str, session: Session = None) -> Job:
         existing_job = self._get_db_job(job_id, session)
         return self._reconstitute_job(existing_job)
 
     @db_user
-    def get_jobs(self, pattern: str = None, session=None) -> List[Job]:
-        db_jobs = session.query(DbJob).all().order_by(DbJob.next_run_time.asc())
+    def get_jobs(self, pattern: str = None, session: Session = None) -> List[Job]:
+        db_jobs = session.query(DbJob).order_by(DbJob.next_run_time.asc()).all()
 
         if pattern is not None:
             compiled_pattern = re.compile(pattern)
@@ -138,7 +139,7 @@ class SqlAlchemyJobStore(JobStore):
         return list(map(lambda db_job: self._reconstitute_job(db_job), db_jobs))
 
     @db_user
-    def contains_job(self, job_id: str, session=None) -> bool:
+    def contains_job(self, job_id: str, session: Session = None) -> bool:
         try:
             self._get_db_job(job_id, session)
             return True
@@ -149,7 +150,7 @@ class SqlAlchemyJobStore(JobStore):
     def _get_db_job(job_id: str, session: Session) -> DbJob:
         existing_job = session.query(DbJob).get(job_id)
         if existing_job is None:
-            raise JobDoesNotExistException
+            raise JobDoesNotExistException(job_id)
         return existing_job
 
     @staticmethod
